// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
import * as path from 'path';
import { FileSystem, JsonFile, JsonSchema, LockFile } from '@rushstack/node-core-library';
import { Utilities } from '../utilities/Utilities';
import { RushUserConfiguration } from '../api/RushUserConfiguration';
import schemaJson from '../schemas/credentials.schema.json';
import { objectsAreDeepEqual } from '../utilities/objectUtilities';
const DEFAULT_CACHE_FILENAME = 'credentials.json';
const LATEST_CREDENTIALS_JSON_VERSION = '0.1.0';
/**
 * @beta
 */
export class CredentialCache /* implements IDisposable */ {
    constructor(cacheFilePath, loadedJson, lockfile) {
        this._modified = false;
        this._disposed = false;
        if (loadedJson && loadedJson.version !== LATEST_CREDENTIALS_JSON_VERSION) {
            throw new Error(`Unexpected ${cacheFilePath} file version: ${loadedJson.version}`);
        }
        this._cacheFilePath = cacheFilePath;
        this._cacheEntries = new Map(Object.entries((loadedJson === null || loadedJson === void 0 ? void 0 : loadedJson.cacheEntries) || {}));
        this._supportsEditing = !!lockfile;
        this._lockfile = lockfile;
    }
    static async initializeAsync(options) {
        let cacheDirectory;
        let cacheFileName;
        if (options.cacheFilePath) {
            cacheDirectory = path.dirname(options.cacheFilePath);
            cacheFileName = options.cacheFilePath.slice(cacheDirectory.length + 1);
        }
        else {
            cacheDirectory = RushUserConfiguration.getRushUserFolderPath();
            cacheFileName = DEFAULT_CACHE_FILENAME;
        }
        const cacheFilePath = `${cacheDirectory}/${cacheFileName}`;
        const jsonSchema = JsonSchema.fromLoadedObject(schemaJson);
        let loadedJson;
        try {
            loadedJson = await JsonFile.loadAndValidateAsync(cacheFilePath, jsonSchema);
        }
        catch (e) {
            if (!FileSystem.isErrnoException(e)) {
                throw e;
            }
        }
        let lockfile;
        if (options.supportEditing) {
            lockfile = await LockFile.acquireAsync(cacheDirectory, `${cacheFileName}.lock`);
        }
        const credentialCache = new CredentialCache(cacheFilePath, loadedJson, lockfile);
        return credentialCache;
    }
    static async usingAsync(options, doActionAsync) {
        await Utilities.usingAsync(async () => await CredentialCache.initializeAsync(options), doActionAsync);
    }
    setCacheEntry(cacheId, entry) {
        this._validate(true);
        const { expires, credential, credentialMetadata } = entry;
        const expiresMilliseconds = (expires === null || expires === void 0 ? void 0 : expires.getTime()) || 0;
        const existingCacheEntry = this._cacheEntries.get(cacheId);
        if ((existingCacheEntry === null || existingCacheEntry === void 0 ? void 0 : existingCacheEntry.credential) !== credential ||
            (existingCacheEntry === null || existingCacheEntry === void 0 ? void 0 : existingCacheEntry.expires) !== expiresMilliseconds ||
            !objectsAreDeepEqual(existingCacheEntry === null || existingCacheEntry === void 0 ? void 0 : existingCacheEntry.credentialMetadata, credentialMetadata)) {
            this._modified = true;
            this._cacheEntries.set(cacheId, {
                expires: expiresMilliseconds,
                credential,
                credentialMetadata
            });
        }
    }
    tryGetCacheEntry(cacheId) {
        this._validate(false);
        const cacheEntry = this._cacheEntries.get(cacheId);
        if (cacheEntry) {
            const result = {
                expires: cacheEntry.expires ? new Date(cacheEntry.expires) : undefined,
                credential: cacheEntry.credential,
                credentialMetadata: cacheEntry.credentialMetadata
            };
            return result;
        }
        else {
            return undefined;
        }
    }
    deleteCacheEntry(cacheId) {
        this._validate(true);
        if (this._cacheEntries.has(cacheId)) {
            this._modified = true;
            this._cacheEntries.delete(cacheId);
        }
    }
    trimExpiredEntries() {
        this._validate(true);
        const now = Date.now();
        for (const [cacheId, cacheEntry] of this._cacheEntries.entries()) {
            if (cacheEntry.expires < now) {
                this._cacheEntries.delete(cacheId);
                this._modified = true;
            }
        }
    }
    async saveIfModifiedAsync() {
        this._validate(true);
        if (this._modified) {
            const cacheEntriesJson = {};
            for (const [cacheId, cacheEntry] of this._cacheEntries.entries()) {
                cacheEntriesJson[cacheId] = cacheEntry;
            }
            const newJson = {
                version: LATEST_CREDENTIALS_JSON_VERSION,
                cacheEntries: cacheEntriesJson
            };
            await JsonFile.saveAsync(newJson, this._cacheFilePath, {
                ensureFolderExists: true,
                updateExistingFile: true,
                ignoreUndefinedValues: true
            });
            this._modified = false;
        }
    }
    dispose() {
        var _a;
        (_a = this._lockfile) === null || _a === void 0 ? void 0 : _a.release();
        this._disposed = true;
    }
    _validate(requiresEditing) {
        if (!this._supportsEditing && requiresEditing) {
            throw new Error(`This instance of ${CredentialCache.name} does not support editing.`);
        }
        if (this._disposed) {
            throw new Error(`This instance of ${CredentialCache.name} has been disposed.`);
        }
    }
}
//# sourceMappingURL=CredentialCache.js.map